/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*************************************************************************************************
 * The content system − a few content classes.
 *************************************************************************************************/
module mde.content.Content;

public import mde.content.IContent;
import util = mde.util;

debug {
    import tango.util.log.Log : Log, Logger;
    private Logger logger;
    static this () {
        logger = Log.getLogger ("mde.content.Content");
    }
}

/** The base for $(I most) content classes.
 *
 * Includes generic callback support, toString implementation and symbol access.
 * 
 * Derived classes should impement:
 * ---
 *  this (char[] symbol, T val = /+ default value +/);
 *  void opAssign (T val);	// assign val, calling callbacks
 *  T opCall ();		// return value
 *  alias opCall opCast;
 * --- */
class AContent : IContent
{
    this (char[] symbol) {
	this.symbol = symbol;
	name_ = symbol;		// provide a temporary name
    }
    
    void name (char[] name, char[] desc = null) {
	name_ = name;
	desc_ = desc;
    }
    
    /** Add a callback. Callbacks are called in the order added. */
    EventContent addCallback (void delegate (AContent) cb) {
	this.cb ~= cb;
	return this;
    }
    /// ditto
    EventContent addCallback (void function (AContent) cb) {
	this.cb ~= util.toDg (cb);
	return this;
    }
    
    char[] toString (uint i) {
	return i == 0 ? "No value"
	: i == 1 ? name_
	: i == 2 ? desc_
	: null;
    }
    
    /// End of an event, e.g. a button release or end of an edit (calls callbacks).
    void endEvent () {
	foreach (dg; cb)
	    dg (this);
    }
    
    final char[] symbol;	// Symbol name for this content
protected:
    char[] name_, desc_;	// name and description
    void delegate (AContent) cb[];
}

/** A generic way to handle a list of type IContent. */
class ContentList : AContent
{
    this (char[] symbol, AContent[] list = null) {
	list_.length = list.length;
	foreach (i,c; list)
	    list_[i] = c;
	super (symbol);
    }
    this (char[] symbol, AContent[char[]] l) {
	list_.length = l.length;
	size_t i;
	foreach (c; l)
	    list_[i++] = c;
	super (symbol);
    }
    
    IContent[] list () {
	return list_;
    }
    
protected:
    final IContent[] list_;
}

/** Created on errors to display and log a message. */
class ErrorContent : IContent
{
    this (char[] msg) {
	this.msg = msg;
    }
    
    char[] toString (uint i) {
	return i == 0 ? msg
	     : i == 1 ? "Error"
	     : null;
    }
    
protected:
    char[] msg;
}

/** A Content with no value but able to pass on an event.
*
* The point being that a button can be tied to one of these. */
alias AContent EventContent;
